#include "CGame.hpp"

#include "CBody_shot.hpp"
#include "CBody_stihacka.hpp"
#include "CBody_talir.hpp"
#include "CBody_ufon.hpp"
#include "CMove_linear.hpp"
#include "CMove_keyboard.hpp"
#include "CMove_magnet.hpp"
#include "CMove_random.hpp"
#include "CFire_forever.hpp"
#include "CFire_keyboard.hpp"
#include "CWeapon_cannon.hpp"
#include "CWeapon_fork.hpp"
#include "CWeapon_magnet.hpp"

#include "CSystem.hpp"

#include <math.h>
#include <stdlib.h>
#include <assert.h>


CGame::CGame() : mFlotila() 
{			
	mTimeAdd = sTimeAdd;
	mLastAdd = 0;
	mLastHard = 0;
	mTarget_up = NULL;
	mTarget_down = NULL;
	setTarget();
	
	// prvotni nastaveni mLastTime
	mLastTime = SDL_GetTicks();
}
//-------------------------------------------------------------------

CGame::~CGame()
{
	if (mTarget_up != NULL) {
		delete mTarget_up;
	}
	if (mTarget_down != NULL) {
		delete mTarget_down;
	}
	// uvolni lode
	for (fleet_t::iterator i = mFlotila.begin(); i != mFlotila.end(); i++) {
		delete (*i);
	}
}
//-------------------------------------------------------------------
/* 
* pridani lodi ovladatelne klavesnici
*/
void
CGame::addPlayer(const COvladani_move &sipky, const COvladani_fire &strelba)
{
	CShip *ship;
	double x = CMove::sPlochaWidth / 2;
	double y = CMove::sPlochaHeight / 1.2;
	
	// hracova lod (letici nahoru)
	// rychlost je zadavana v pocet pixelu / za sekundu (hrac ma 200 pro x i y)
	ship = new CShip(x, y, true,
			new CBody_stihacka(),
			new CMove_keyboard(200, 200, sipky),
			new CFire_keyboard(strelba) );
	ship->addWeapon( new CWeapon_cannon() );
	mFlotila.push_front(ship);
}
//-------------------------------------------------------------------
/* 
* nalezeni vhodnych cilu pro magneticke navadeni
* naleznou se lode s nejvetsi energoskou co jsou tu nejdele
*/
	void
CGame::setTarget() 
{
	int power_up = -1;
   	int	power_down = -1;
	
	delete mTarget_up;
	delete mTarget_down;	
	mTarget_up = NULL;
	mTarget_down = NULL;

	// na zacatku seznamu jsou nemladsi lode, postupne se prokousem k nejstarsim
	for (fleet_t::iterator i = mFlotila.begin(); i != mFlotila.end(); i++) {
		if ( (*i)->getUp() ) {
		   	if ( (*i)->getPower() >= power_up) {						
				// pokud se rovnaji pak rozhodne nahoda ktery se vezmeme
				if ( (*i)->getPower() != power_up || (CSystem::random(2)) == 1) {															
					// prozitivne pridelime jen ukazatel
					mTarget_up = (*i);
					power_up = (*i)->getPower();
				}
			}
		} else {
		   	if ( (*i)->getPower() >= power_down) {										
				// pokud se rovnaji pak rozhodne nahoda ktery se vezmeme
				if ( (*i)->getPower() != power_down || (CSystem::random(2)) == 1) {															
					// prozitivne pridelime jen ukazatel
					mTarget_down = (*i);
					power_down = (*i)->getPower();
				}
			}
		}
	}

	// jestli existuji cile, pak pro ne poradne uvolni vlastni pamet
	if (mTarget_up != NULL) {
		mTarget_up = new CXY(*mTarget_up);
	}
	if (mTarget_down != NULL) {
		mTarget_down = new CXY(*mTarget_down);
	}
}
//-------------------------------------------------------------------

	void
CGame::round(Uint8 *keys) 
{		
	double deltaTime = getDeltaTime();
	Uint32 ticks = SDL_GetTicks();
	fleet_t *naboje;
	CXY *target;

	// pohni vsemi
	for (fleet_t::iterator i = mFlotila.begin(); i != mFlotila.end();) {
		// nastavime mu odpovidajici target
		if ( (*i)->getUp() ) {
			target = mTarget_down;
		} else {
			target = mTarget_up;
		}
		
		if ( (*i)->letMove(keys, deltaTime, target) ) {
			// vyletnul definitivne z hraci plochy
			// nezapomenout na zavolan "delete" na ukazatel
			delete (*i);
			i = mFlotila.erase(i);
		} else {
			i++;
		}
	}

	// vysetreni narazu
	bouracky();

	// nastaveni cilu
	setTarget();

	// vystrel vsemi
	for (fleet_t::iterator i = mFlotila.begin(); i != mFlotila.end(); i++) {
		// nastavime mu odpovidajici target
		if ( (*i)->getUp() ) {
			target = mTarget_down;
		} else {
			target = mTarget_up;
		}

		naboje = (*i)->letFire(keys, ticks, target);
		if (naboje != NULL) {							
			// pozn. vlozenim do std::list se iteratory nezmeni
			mFlotila.insert(mFlotila.begin(), naboje->begin(), naboje->end());
			// zavolanim delete se jen uvolni seznam,
			// na ukazatele v nem nikdo delete nevola
			delete naboje;			
		}
	}
	
	// moznost pridani novych lodi
	rozvrh(deltaTime);	
}

//-------------------------------------------------------------------
	void
CGame::bouracky() 
{
	// vysetreni narazu
	for (fleet_t::iterator i = mFlotila.begin(); i != mFlotila.end();) {
		i = vnitrni(i);
	}
}


	fleet_t::iterator 
CGame::vnitrni(fleet_t::iterator i)
{
	// predpocitani hodnot pro 'i'
	CShip *a = (*i);
	bool iUp = a->getUp();
		
	// provadime pro 'j' > 'i'
	fleet_t::iterator j = i;
	for (j++; j != mFlotila.end(); ) {
		if ( (*j)->getUp() != iUp ) {
			switch ( crash(a, *j) ) {
				case 3:
					// do 'j' se priradi iterator na nasledujici prvek
					delete (*j);
					j = mFlotila.erase(j);
					// neni break!
				case 1:
					delete (a);
					i = mFlotila.erase(i);
					return i;
					break;
				case 2:
					delete (*j);
					j = mFlotila.erase(j);
					break;
				case 0:
					// zvetsime 'j' manualne
					j++;
					// bouracka se nekonala
					break;
				default:
					assert("Neznamy dusledek narazu" == NULL);
			}
		} else {
			j++;
		}
	}
	return ++i;
}	

//-------------------------------------------------------------------
/*
* !!! nejvice volana funkce !!!
* 
* vysetreni a osetreni narazu
* vraci binarne kdo to neprezil
* 3...oba
* 2...'j' neprezil
* 1...'i' neprezil
* 0... nic se nestalo
 */
	int
CGame::crash(CShip *a, CShip *b) 
{
	// kolik pixelu musi byt v sobe zaklineni
	static const int vule = 6;
	
	// vypocet zjisteni narazu
	// jestlize absolutni hodnota rozdil stredu je mensi nez soucet polovin sirek
	if ( ( abs( static_cast<int>(a->getX() - b->getX()) ) <= a->getPulW() + b->getPulW() - vule ) &&
			( abs( static_cast<int>(a->getY() - b->getY()) ) <= a->getPulH() + b->getPulH() - vule )) {

		// zalohujeme si energosky nez je snizime,
		// abychom je mohli odecist od obou
		int aPower = a->getPower();
		int bPower = b->getPower();

		bool prvni_kach =  a->crash(bPower);
		bool druhy_kach = b->crash(aPower);

		// pozn. strely kachnou vzdy
		if (prvni_kach && !druhy_kach) {
			// predej zbrane druhemu
			b->loot(a);

			return 1;
		} else {
			if (druhy_kach && !prvni_kach) {
				// predej zbrane prvnimu
				a->loot(b);
				return 2;			
			} else {	
				// oba kachli			
				return 3;	
			}		
		}			
	}

	// nikdo nekachnul
	return 0;
}	

//-------------------------------------------------------------------
	void 
CGame::draw(SDL_Surface *surface) 
{
	for (fleet_t::iterator i = mFlotila.begin(); i != mFlotila.end(); i++) {
		(*i)->draw(surface);
	}
}


//-------------------------------------------------------------------
/*
* deltaTime = kolik ubehlo sekund za posledni kolo
 */
	double
CGame::getDeltaTime() 
{
	double deltaTime;
	Uint32 current = SDL_GetTicks();

	deltaTime = current - mLastTime;
	deltaTime /= 1000.0;
	mLastTime = current;
	return deltaTime;	
}

//-------------------------------------------------------------------
/*
* pridava nove lode do hry
 */
	void
CGame::rozvrh(double deltaTime)
{
	mLastAdd += deltaTime;
	if (mLastAdd > mTimeAdd) {
		// pridavame nerovnomerne, vice lodi co leti dolu
		addShip(CSystem::random(4) == 0);
		mLastAdd = 0;
	}
	// pomale zvysovani obtiznosti
	mLastHard += deltaTime;
	if (mLastHard > sTimeAdd) {
		mLastHard = 0;
		mTimeAdd *= sZhustovani;
	}
}

//-------------------------------------------------------------------
/* 
* vlozi nahodne vygenerovanou lod
 */
	void 
CGame::addShip(bool up)
{
	CShip *ship;
	CBody *body;
	CMove *move;
	CFire *fire;

	// body ---------------------------------
	switch ( CSystem::random(3) /* kolik typu */) {
		case 0:
			body = new CBody_stihacka();
			break;
		case 1:
			body = new CBody_talir();
			break;
		case 2:
			body = new CBody_ufon();
			break;
		default:
			assert("blbe body\n" == NULL);
			exit(1);
	}

	// move ---------------------------------
	double velX = 100 - CSystem::random(200);
	double velY = 1 + CSystem::random(200);
	if (up) {
		velY = -velY;
	}	
	switch (CSystem::random(3) /* kolik typu */) {
		case 0:
			move = new CMove_linear(velX, velY);
			break;
		case 1:
			move = new CMove_random(velX, velY);
			break;
		case 2:
			move = new CMove_magnet(velX, velY);
			break;
		default:
			assert("blbe move\n" == NULL);		
			exit(1);
	}

	// fire ---------------------------------
	switch ( CSystem::random(1) /* kolik typu */) {
		case 0:
			fire = new CFire_forever();
			break;
		default:
			assert("blbe fire\n" == NULL);		
			exit(1);
	}

	// lod beze zbrani -----------------------
	// vypocteme vhodne souracnice
	double x = 4 * body->getPulW() + CSystem::random( CMove::sPlochaWidth - 4 * body->getPulW() );
	double y;
	if (up) {
		y =	2 * body->getPulH() + CMove::sPlochaHeight;
	} else {
		y = - 2 * body->getPulH(); 
	}
	ship = new CShip(x, y, up, body, move, fire);


	// weapon ---------------------------------
	for (int i = 0; i < CSystem::random(5) /* max pocet zbrani */; i++) {
		CWeapon *weapon;

		switch ( CSystem::random(3) /* kolik typu */) {
			case 0:
				weapon = new CWeapon_cannon();
				break;
			case 1:
				weapon = new CWeapon_fork();
				break;
			case 2:
				weapon = new CWeapon_magnet();
				break;
			default:
				assert("blbe weapon\n" == NULL);		
				exit(1);
		}
		ship->addWeapon(weapon);
	}

	// ulozeni do seznamu
	mFlotila.push_front(ship);	
}

