#include "CSystem.hpp"
#include "CGame.hpp"
#include "CBg.hpp"

#include <stdlib.h>
#include <assert.h>

#include "SDL.h"
#include "SDL_mixer.h"

/*
* globalni promenne
 */
// zobrazovaci plocha
SDL_Surface *screen;
// musika
Mix_Music *music = NULL;
// udaje o hre
CGame *game;

/*
* konstanty ovladani
* pozor na nedokonalosti klavesnice, aby se co nejmene blokovali
 */
// sturctura: mDown, mUp, mLeft, mRight;
COvladani_move sipky_player1 = {SDLK_DOWN, SDLK_UP, SDLK_LEFT, SDLK_RIGHT};
COvladani_fire strelba_player1 = {SDLK_RSHIFT};
// druhy hrac
COvladani_move sipky_player2 = {SDLK_s, SDLK_w, SDLK_a, SDLK_d};
COvladani_fire strelba_player2 = {SDLK_LSHIFT};
// zda se muze pripojit druhy hrac
bool play2Volno = true;




/*
* funkce provedena na konci programu
 */
	void
totalQuit() 
{
	Mix_CloseAudio();
	SDL_Quit();
}


/*
* inicializace SDL a videa
 */
	void 
videoInit() 
{	
	CSystem::echo("Inicializuji SDL ...");
	if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0 ) {
		CSystem::problemSDL();
	}
	atexit(totalQuit);

	// nastavime nadpis okna
	SDL_WM_SetCaption("ships - by fidlej & SDL", "ships");

	CSystem::echo("Inicializuji Video mod ...");
	screen = SDL_SetVideoMode(CMove::sPlochaWidth, CMove::sPlochaHeight, 16, SDL_SWSURFACE|SDL_ANYFORMAT);	
	if (screen == NULL) {
		CSystem::problemSDL();
	}

	CSystem::echo("SDL je nyni pripraveno.");
}	

/*
* funkce kterou necham vykonat po skonceni skladby
 */
	void 
musicDone() 
{
	if (music != NULL) {
		Mix_HaltMusic();
		Mix_FreeMusic(music);
	}
	// nahodne vybrani skladby
	switch (CSystem::random(3) /* kolik sladeb */) {
		case 0:	
			music = Mix_LoadMUS("music/mod.apoplexy");
			break;
		case 1:	
			music = Mix_LoadMUS("music/mod.chiptune");
			break;
		case 2:	
			music = Mix_LoadMUS("music/mod.cream_of_the_earth");
			break;
		default:
			assert("blba musika" == NULL);
	}
	// 0 ... znaci bez opakovani
	Mix_PlayMusic(music, 0);
	Mix_HookMusicFinished(musicDone);
}


/*
* zapnuti musiky
* jestli nekdo pouziva /dev/dsp (`lsof /dev/dsp`)
* tak sice lze hrat se zvukem, ale je posunuty za obrazem 
* (system ho jeste jednou mixuje
 */
	void
audioInit()
{
	// nastaveni kvality audia
	int audio_rate = 44100; //22050
	Uint16 audio_format = AUDIO_S16SYS; /* 16-bit signed */
	int audio_channels = 2; //stereo
	// buffer nez se zvuk skipne protoze se nestihl
	int audio_buffers = 1024; // 4096 

	CSystem::echo("Inicializuji SDL_mixer ...");
	if( Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) < 0 ) {
		CSystem::echo("Unable to open audio!");
		CSystem::problemSDL();
	}

	// nacteni nove skladby
	musicDone();
}

/*
* moje obsluha stisknute klavesy
* vraci zda ma program skoncit ci ne
 */
	bool
keyEvent(SDLKey sym)
{
	// zda mame fullscreen
	static bool fullScreen = false;

	switch (sym) {
		case SDLK_ESCAPE:
			return true;
		case SDLK_f:
			SDL_WM_ToggleFullScreen(screen);
			fullScreen = !fullScreen;
			// pri fullScreenu schovame cursor
			if (fullScreen) {
				SDL_ShowCursor(SDL_DISABLE);
			} else {
				SDL_ShowCursor(SDL_ENABLE);
			}
			break;
		case SDLK_2:
			// pridame lod s timto ovladanim
			if (play2Volno) {
				game->addPlayer(sipky_player2, strelba_player2);
				play2Volno = false;
			}				
			break;		
		case SDLK_KP_MINUS:
			{
				// snizime hlasitost
				int soucasnaMusic = Mix_VolumeMusic(-1);
				int soucasnaZvuk = Mix_Volume(-1, -1);
				
				Mix_VolumeMusic(soucasnaMusic - 10);
				// -1 znamena na vsech kanalech
				Mix_Volume(-1, soucasnaZvuk -10);
				break;
			}
		case SDLK_KP_PLUS:
			{
				// zvysime hlasitost
				int soucasnaMusic = Mix_VolumeMusic(-1);
				int soucasnaZvuk = Mix_Volume(-1, -1);
				
				Mix_VolumeMusic(soucasnaMusic + 10);
				// -1 znamena na vsech kanalech
				Mix_Volume(-1, soucasnaZvuk +10);
				break;
			}
		default:
			//ostatni si vytahnou hraci z pole 'keys'
			break;
	}	
	return false;
}	

//----------------------------------------------------------------------
/*
* funkce main
 */
	int 
main() 
{
	// inicializace SDL a video modu
	videoInit();
	// inicializace Audia a spusteni skladby
	audioInit();
	// inicializace pozadi
	CBg *pozadi = new CBg("bg/space_pas.bmp");
	// nastaveni pocatku hry
	game = new CGame();
	// pocatecni zmacnuti klaves
	Uint8 *keys = SDL_GetKeyState(NULL);

	// vlozeni 1. hrace
	game->addPlayer(sipky_player1, strelba_player1);

	// pro zjisteni poctu FPS
	Uint32 casStartu = SDL_GetTicks();
	int pocetFramu = 0;

	bool done = false;	
	while (done == false) {
		SDL_Event event;


		while ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_QUIT:
					done = true;
					break;
				case SDL_KEYDOWN:
					// osetreni stisknuti klavesy
					done = keyEvent(event.key.keysym.sym);

					// neni break !
				case SDL_KEYUP:
					// update pole se stisknutymi klavesy
					keys = SDL_GetKeyState(NULL);
					break;
			}
		}

		game->round(keys);
		pozadi->draw(screen);
		game->draw(screen);
		// update obrazovky
		SDL_Flip(screen);
		pocetFramu++;
	}

	Uint32 doba = SDL_GetTicks() - casStartu; 
	printf("FPS: %d\n", (pocetFramu*1000) / doba );	
	printf("Herni doba:\n %g minut\n", doba / 60000.0 );


	// ? musi se to uvolnovat ?
	delete game;
	delete pozadi;

	// konec	
	exit(0);
}







